# Project:    PiView
# Filename:   GPU.py
# Location:   ./piview
# Author:     Adrian Gould <adrian.gould@nmtafe.wa.edu.au>
# Created:    10/04/21
#
# This file provides the following features, methods and associated supporting
# code:
# - temperature

import subprocess


class GPU:
    @staticmethod
    def temperature():
        """
        Requests the GPU temperature from the thermal zone details

        :rtype: string
        :return: GPU temperature in degrees Celsius to 2DP
        """
        try:
            temp = subprocess.check_output(
                ['cat', '/sys/class/thermal/thermal_zone0/temp'])
            temp = float(temp) / 1000
        except:
            temp = -273.16  # use absolute zero for no reading
        temp = round(temp, 2)
        return temp



# Project:    PiView
# Filename:   Network.py
# Location:   ./piview
# Author:     Adrian Gould <adrian.gould@nmtafe.wa.edu.au>
# Created:    10/04/21
#
# This file provides the following features, methods and associated
# supporting code:
# - host name
# - interface names
# - ip addresses
# - mac addresses

import os
from socket import gethostname


class Network:
    @staticmethod
    def host_name():
        """Provide the host name to the user

        :rtype: string
        :return: The host name of the Pi
        """
        # return platform.node()
        # return platform.uname()[1]
        return gethostname()

    @staticmethod
    def eth_name(_type=None):
        """Provide the Ethernet interface name

        :param _type: string, possible options are: enx or eth
        :rtype: string
        :return: The network interface name
        """
        options = ['enx', 'eth']
        interface = None
        if _type == 'w':
            options = ['wla']
        if _type == 'l':
            options = ['lo']
        try:
            for root, dirs, files in os.walk('/sys/class/net'):
                for dir in dirs:
                    for option in options:
                        if dir[:3] == option:
                            interface = dir
        except:
            interface = "None"
        return interface

    @staticmethod
    def mac(interface='eth0'):
        """Provides the hardware MAC address for the interface requested

        Default is eth0

        :param interface: string, the interface name to query
        :rtype: string
        :return: Mac address of the selected interface
        """
        # Return the MAC address of named Ethernet interface
        try:
            line = open('/sys/class/net/%s/address' % interface).read()
        except:
            line = "None"
        return line[0:17]

    @staticmethod
    def ip(interface='eth0'):
        """Provide IP Address from the named interface

        Default is eth0

        Uses the ifconfig command to create a text file, then processes this
        file to obtain the IP address.

        :param interface: string, the interface to obtain the IP address for
        :rtype: string
        :return: IPv4 address of the selected interface
        """
        try:
            filename = 'ifconfig_' + interface + '.txt'
            os.system('ifconfig ' + interface + ' > /home/pi/' + filename)
            f = open('/home/pi/' + filename, 'r')
            skip_line = f.readline()  # skip 1st line
            line = f.readline()  # read 2nd line
            line = line.strip()
            f.close()

            if line.startswith('inet '):
                a, b, c = line.partition('inet ')
                a, b, c = c.partition(' ')
                a = a.replace('addr:', '')
            else:
                a = 'None'

            return a

        except:
            return 'Error'



# Project:    PiView
# Filename:   Hardware.py
# Location:   ./piview
# Author:     Adrian Gould <adrian.gould@nmtafe.wa.edu.au>
# Created:    10/04/21
#
# This file provides the following features, methods and associated supporting
# code:
# - bt (Bluetooth status)
# - camera (camera supported, detected)
# - spi (SPI Bus status)
# - i2c (I2C status)
#
# TODO: Cache the lsmod for user defined time - store in a dictionary?
# TODO: Use the cached lsmod, unless force update required
# TODO: Dry out bt/spi/i2c - to remove the repeated filters

import subprocess


class Hardware:
    @staticmethod
    def bt():
        """
        Check if Bluetooth module is enabled

        :rtype: boolean
        :return: True|False
        """
        bt = False
        try:
            c = subprocess.Popen("lsmod", stdout=subprocess.PIPE)
            gr = subprocess.Popen(["grep", "bluetooth"], stdin=c.stdout,
                                  stdout=subprocess.PIPE)
            output = gr.communicate()[0]
            if output[:9] == 'bluetooth':
                bt = True
        except:
            pass
        return bt

    @staticmethod
    def spi():
        """
        Check if SPI bus is enabled by checking for spi_bcm2 modules

        :rtype: boolean
        :return: True|False
        """
        spi = False
        try:
            c = subprocess.Popen("lsmod", stdout=subprocess.PIPE)
            gr = subprocess.Popen(["grep", "spi_bcm2"], stdin=c.stdout,
                                  stdout=subprocess.PIPE)
            output = gr.communicate()[0]
            if output[:8] == 'spi_bcm2':
                spi = True
        except:
            pass
        return spi

    @staticmethod
    def i2c():
        """
        Check if I2C bus is enabled by checking for i2c_bcm2 modules

        :rtype: boolean
        :return: True|False
        """
        i2c = False
        try:
            c = subprocess.Popen("lsmod", stdout=subprocess.PIPE)
            gr = subprocess.Popen(["grep", "i2c_bcm2"], stdin=c.stdout,
                                  stdout=subprocess.PIPE)
            output = gr.communicate()[0]
            if output[:8] == 'i2c_bcm2':
                i2c = True
        except:
            pass
        return i2c

    @staticmethod
    def camera():
        """
        Check if camera is enabled and present

        :rtype: dictionary
        :return: Details in form {"supported": boolean, "detected": boolean}
        """
        camera = {"supported": None, "detected": None}
        try:
            c = subprocess.Popen(["/opt/vc/bin/vcgencmd", "get_camera"],
                                 stdout=subprocess.PIPE)
            output = c.communicate()[0]
            supported = output[10] == ord("1")
            detected = output[21] == ord("1")
            camera = {"supported": supported, "detected": detected}
        except:
            pass
        return camera



# Project:    PiView
# Filename:   Utils.py
# Location:   ./piview
# Author:     Adrian Gould <adrian.gould@nmtafe.wa.edu.au>
# Created:    10/04/21
#
# This file provides the following features, methods and associated
# supporting code:
# - draw a line using a provided string
# - format a number (bytes) into Bytes, KB, MB, GB,...
# - random number (percentage) between max and min percentages

import random


class Utils:
    @staticmethod
    def draw_line(characters="-=-", length=40):
        """
        Draw a line of characters using a given character and length

        :param characters: string, the character(s) to draw with
        :param length: integer), the length of the line
        :rtype: string
        :return: A string of exactly length characters
        """
        characters_length = len(characters)
        repeats = length // characters_length
        extras = length % characters_length

        line = characters * repeats

        for count in range(extras):
            next_character = characters[count]
            line = f"{line}{next_character}"

        return line

    # Original found in:
    # https://stackoverflow.com/questions/12523586/python-format-size
    # -application
    # -converting-b-to-kb-mb-gb-tb/37423778
    @staticmethod
    def format_bytes(size=0, style=None):
        """
        Formats the given value into Bytes, Kilobytes, Megabytes, ...

        Using Byte shorthand by default - B, KB, MB, ...

        The style may be:

        - None | short | s  -- Short labels
        - long | l          -- Long labels

        If style is anything other than above, then defaults to long format.

        :param size: integer, defaults to 0
        :param style: string, defaults to None
        :rtype: tuple
        :return: (float, string)
        """
        power = 2 ** 10  # 2**10 = 1024
        n = 0
        short_labels = {0: '', 1: 'K', 2: 'M', 3: 'G', 4: 'T',
                        5: 'P', 6: 'E', 7: 'Y'}
        long_labels = {0: '', 1: 'Kilo', 2: 'Mega', 3: 'Giga',
                       4: 'Tera', 5: 'Peta', 6: 'Exa', 7: 'Yotta'}
        short_end = "B"
        long_end = "bytes"
        while size >= power:
            size /= power
            n += 1

        if style in [None, "short", 's']:
            power_labels = short_labels.copy()
            suffix = short_end
        else:
            power_labels = long_labels.copy()
            suffix = long_end

        return size, power_labels[n] + suffix

    @staticmethod
    def random_percentage(min_percentage=0, max_percentage=100):
        """
        This function returns a random percentage.
        Useful for simulations when developing monitoring dashboards


        :param min_percentage: float, minimum value to return, default 0.0
        :param max_percentage: float, maximum to return, default 100.0
        :rtype: float
        :return: A random CPU load value between 0% and 100% to 1DP
        """
        load = random.gauss(55, 10)
        if load < min_percentage:
            return 0.0
        elif load > max_percentage:
            return 100.0
        else:
            return round(load, 1)



# Project:    PiView
# Filename:   Host.py
# Location:   ./piview
# Author:     Adrian Gould <adrian.gould@nmtafe.wa.edu.au>
# Created:    10/04/21
#
# This file provides the following features, methods and associated supporting
# code:
# - get boot time
# - get model
# - get revision
# - get serial
# - get uptime
#
# The following may be deprecated at some point:
# - name

from datetime import datetime, timedelta
from socket import gethostname
from sys import version_info

import psutil


class Host:
    @staticmethod
    def boot_time():
        """
        Determines the time the device was started

        :rtype: datetime
        :return: How long ago the Pi was booted
        """
        booted_at = datetime.fromtimestamp(psutil.boot_time())
        return booted_at

    @staticmethod
    def model():
        """
        Provide Pi Model Details

        Extracts the details from the device tree model file

        :rtype: string
        :return: The model name and other identifying details
        """
        try:
            my_model = open('/proc/device-tree/model').readline()
        except:
            my_model = "Error"

        return my_model

    @staticmethod
    def name():
        """
        Provides the host name to the user

        :rtype: string
        :return: The host name of the Pi
        """
        return gethostname()

    @staticmethod
    def python():
        """
        Get current Python version

        :rtype: string
        :return: A string containing the version of python that is being used
        """
        pythonv = '.'.join([str(x) for x in version_info[:3]])
        return pythonv

    @staticmethod
    def revision():
        """
        Provide board revision details

        The details are extracted from the cpu info file

        :rtype: string
        :return: The revision number of the Pi motherboard
        """
        my_revision = "Error"
        try:
            f = open('/proc/cpuinfo', 'r')
            for line in f:
                if line[0:8] == 'Revision':
                    my_revision = line[11:-1]
            f.close()
        except:
            my_revision = "Error"

        return my_revision

    @staticmethod
    def serial():
        """
        Provide the Serial Number of the Pi CPU

        The details are extracted from the cpu info file

        :rtype: string
        :return: The Pi's serial number
        """
        my_cpu_serial = "Error"
        try:
            f = open('/proc/cpuinfo', 'r')
            for line in f:
                if line[0:6] == 'Serial':
                    my_cpu_serial = line[10:26]
            f.close()
        except:
            my_cpu_serial = "Error"

        return my_cpu_serial

    @staticmethod
    def uptime():
        """
        Determines the amount of time the device has been running for in
        seconds

        :rtype: float
        :return: The time that the Pi has been 'up' for
        """
        booted_at = Host.boot_time()
        current_at = datetime.now()
        uptime_seconds = (current_at - booted_at).total_seconds()
        uptime = timedelta(seconds=uptime_seconds)
        return uptime



# Project:    PiView
# Filename:   Storage.py
# Location:   ./piview
# Author:     Adrian Gould <adrian.gould@nmtafe.wa.edu.au>
# Created:    10/04/21
#
# This file provides the following features, methods and associated
# supporting code:
# - total ram and free ram (tuple)
# - total disk storage and free storage space (tuple)
# - disc storage for all disks connected to Pi (Total and Free) as a list or
# dictionary (To be determined)

import subprocess


class Storage:
    @staticmethod
    def ram():
        """
        Provide the total RAM and free RAM to the user as a tuple

        :rtype: tuple
        :return: (Total, Free)

        """
        output = subprocess.check_output(['free', '-m'])
        lines = output.splitlines()
        line = str(lines[1])
        ram = line.split()
        # total/free
        return ram[1], ram[3]

    @staticmethod
    def disc():
        """
        Provide the total disc space and the disc space that is free

        :rtype: tuple
        :return: (Total, Free)

        """
        output = subprocess.check_output(['df', '-h'])
        lines = output.splitlines()
        line = str(lines[1])
        disk = line.split()
        # total/free
        return disk[1], disk[3]

    @staticmethod
    def all_discs():
        """
        Provide the user with the storage space (Total, Free) for each
        disc attached to the Pi as a dictionary.

        The dictionary will have: "disc name" : (total storage, free storage)
        for each disc.

        :return disc stats: dictionary of tuples
        """
        disc_stats = {}
        # TODO: Create list of all drives and the total and free space for each
        # For each disc:
        #   obtain free space and total space
        #   add "disk_name" :(total, free) to 'disc_stats' dictionary
        return disc_stats

    @staticmethod
    def disk():
        """
        Alias for disc()

        see :func:`~piview.Storage.disc`
        """
        return Storage.disc()

    @staticmethod
    def all_disks():
        """
        Alias for all_discs()

        see :func:`~piview.Storage.all_discs`
        """
        return Storage.all_discs()



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md') as history_file:
    history = history_file.read()

requirements = ['psutil']

test_requirements = []

setup(
    author="Adrian Gould",
    author_email='adrian.gould@nmtafe.wa.edu.au',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Open Software License 3.0 (OSL-3.0)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Raspberry Pi System/Hardware Information",
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description_content_type="text/markdown",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='piview',
    name='piview',
    packages=find_packages(include=['piview', 'piview.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/AdyGCode/piview',
    version='2.0.4',
    zip_safe=False,
)



# Project:    PiView
# Filename:   CPU.py
# Location:   ./piview
# Author:     Adrian Gould <adrian.gould@nmtafe.wa.edu.au>
# Created:    10/04/21
#
# This file provides the following features, methods and associated supporting
# code:
# - speed
# - maximum load
# - temperature

import subprocess

import psutil

from piview.Utils import Utils


class CPU:
    @staticmethod
    def speed():
        """
        Get the CPU frequency using the vcgencmd on Linux based systems

        If frequency cannot be determined, returns -1

        :rtype: integer
        :return: The CPU frequency in MHz
        """
        try:
            output = subprocess.check_output(
                ['vcgencmd', 'get_config', 'arm_freq'])
            output = output.decode()
            lines = output.splitlines()
            line = lines[0]
            freq = line.split('=')
            freq = float(freq[1])
        except:
            freq = -1
        return freq

    @staticmethod
    def max_load(random=False):
        """
        This function returns the maximum "CPU load" across all CPU cores,
        or -1 if no value determined.

        Providing a random=True parameter value will return a random value if
        the actual CPU load can't be determined.

        :param random: boolean default False
        :rtype: float
        :return: The maximum CPU Load in range 0-100
        """
        value = -1
        if psutil is not None:
            value = max(psutil.cpu_percent(percpu=True))
        if random:
            value = Utils.random_percentage()
        return float(value)

    @staticmethod
    def temperature():
        """
        Requests the CPU temperature from the vcgencmd returning the
        result to the caller as a floating point value to 2DP

        If no value can be determined, uses -999.99 as the
        returned "error" value.

        :rtype: float
        :return: The CPU temperature in degrees Celsius
        """
        try:
            temp = subprocess.check_output(['vcgencmd', 'measure_temp'])
            temp = float(temp[5:-3])
        except:
            temp = -999.99
        temp = round(temp, 2)
        return temp


    @staticmethod
    def temperature_k():
        """Provide temperature of the CPU in degrees Kelvin

        :return: CPU Temperature in Kelvin
        """
        temp_k =  CPU.temperature()
        if temp_k == -999.99:
            return temp_k
        return temp_k - 273.16
